unit rwincontrolclient;

{$mode objfpc}{$H+}

{*******************************************************************************
*  == LICENSE ==                                                               *
*  (C) 2009 Covac Software. You may copy, store, modify and sell this software *
*      as long as you keep copyright comments in the source code.              *
*******************************************************************************}

interface

uses rwincontrol;

type
  TRCreate = function(ClassName:TRString):TRHandle;
  TRFree = procedure(Handle:TRHandle);
  TRSetParent =  procedure(Handle:TRHandle; Parent:TRHandle);
  TRGetParent =  function(Handle:TRHandle):TRHandle;
  TRSetStrProp = procedure(Handle:TRHandle; Prop:TRString; Value:TRString);
  TRGetStrProp = function(Handle:TRHandle; Prop:TRString):TRString;
  TRSetIntProp = procedure(Handle:TRHandle; Prop:TRString; Value:Integer);
  TRGetIntProp = function(Handle:TRHandle; Prop:TRString):Integer;
  TRSetObjProp = procedure(Handle:TRHandle; Prop:TRString; Value:TRHandle);
  TRGetObjProp = function(Handle:TRHandle; Prop:TRString):TRHandle;
  { Demo and base TRWinControl class }
  TRWinControl=class(TObject)
    private
      // property handlers (application)
      procedure _setStrProp(Prop,Value:String);
      function  _getStrProp(Prop:String):String;
      procedure _setIntProp(Prop:String; Value:Integer);
      function  _getIntProp(Prop:String):Integer;
      procedure _setObjProp(Prop:String; Value:TRHandle);
      function  _getObjProp(Prop:String):TRHandle;
      // property handlers (library)
      procedure SetLeft(Value:Integer);
      procedure SetTop(Value:Integer);
      procedure SetWidth(Value:Integer);
      procedure SetHeight(Value:Integer);
      function GetLeft:Integer;
      function GetTop:Integer;
      function GetWidth:Integer;
      function GetHeight:Integer;
    protected
      FRHandle:TRHandle;
    public
      constructor Create(AOwner: TRHandle); overload;
      destructor Destroy; override;
      property Left:Integer read GetLeft write SetLeft;
      property Top:Integer read GetTop write SetTop;
      property Width:Integer read GetWidth write SetWidth;
      property Height:Integer read GetHeight write SetHeight;
  end;

var
  RCreate:TRCreate;
  RFree:TRFree;
  RSetParent:TRSetParent;
  RGetParent:TRGetParent;
  RSetStrProp:TRSetStrProp;
  RGetStrProp:TRGetStrProp;
  RSetIntProp:TRSetIntProp;
  RGetIntProp:TRGetIntProp;
  RSetObjProp:TRSetObjProp;
  RGetObjProp:TRGetObjProp;

procedure RProcessInterface(intr:TRWinControlInterface);


implementation

{ Initialize interface }

procedure RProcessInterface(intr:TRWinControlInterface);
begin
  RCreate:= TRCreate( intr.Create);
  RFree:= TRFree( intr.Free);
  RSetParent:= TRSetParent( intr.Parent.Setter);
  RGetParent:= TRGetParent( intr.Parent.Getter);
  RSetStrProp:= TRSetStrProp( intr.StrProp.Setter);
  RGetStrProp:= TRGetStrProp( intr.StrProp.Getter);
  RSetIntProp:= TRSetIntProp( intr.IntProp.Setter);
  RGetIntProp:= TRGetIntProp( intr.IntProp.Getter);
  RSetObjProp:= TRSetObjProp( intr.ObjProp.Setter);
  RGetObjProp:= TRGetObjProp( intr.ObjProp.Getter);
end;

{ Demo and base TRWinControl class }

constructor TRWinControl.Create(AOwner: TRHandle);
begin
  FRHandle:= RCreate( StrToRstr( 'TWinControl'));
  RSetObjProp( FRHandle, StrToRstr( 'Owner'), AOwner);
end;

destructor TRWinControl.Destroy;
begin
  RFree( FRHandle);
end;

procedure TRWinControl._setStrProp(Prop,Value:String);
begin
  RSetStrProp( FRHandle, StrToRstr( Prop), StrToRstr( Value));
end;

procedure TRWinControl._setIntProp(Prop:String; Value:Integer);
begin
  RSetIntProp( FRHandle, StrToRstr( Prop), Value);
end;

procedure TRWinControl._setObjProp(Prop:String; Value:TRHandle);
begin
  RSetObjProp(FRHandle,StrToRstr(Prop),Value);
end;

function TRWinControl._getStrProp(Prop:String):String;
begin
  Result:=RstrToStr(RGetStrProp(FRHandle,StrToRstr(Prop)));
end;

function TRWinControl._getIntProp(Prop:String):Integer;
begin
  Result:=RGetIntProp(FRHandle,StrToRstr(Prop));
end;

function TRWinControl._getObjProp(Prop:String):TRHandle;
begin
  Result:=RGetObjProp(FRHandle,StrToRstr(Prop));
end;


procedure TRWinControl.SetLeft(Value:Integer);
begin
  _setIntProp('Left',Value);
end;

procedure TRWinControl.SetTop(Value:Integer);
begin
  _setIntProp('Top',Value);
end;

procedure TRWinControl.SetWidth(Value:Integer);
begin
  _setIntProp('Width',Value);
end;

procedure TRWinControl.SetHeight(Value:Integer);
begin
  _setIntProp('Height',Value);
end;

function TRWinControl.GetLeft:Integer;
begin
  Result:=_getIntProp('Left');
end;

function TRWinControl.GetTop:Integer;
begin
  Result:=_getIntProp('Top');
end;

function TRWinControl.GetWidth:Integer;
begin
  Result:=_getIntProp('Width');
end;

function TRWinControl.GetHeight:Integer;
begin
  Result:=_getIntProp('Height');
end;

end.

